@@ -487,84 +487,141 @@ INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 le
     return adjustment;
 }
 
-void freetype_get_glyph_bbox(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, BOOL nohint, RECT *ret)
+void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
 {
     FTC_ImageTypeRec imagetype;
     FT_BBox bbox = { 0 };
     FT_Glyph glyph;
 
-    imagetype.face_id = fontface;
+    imagetype.face_id = bitmap->fontface;
     imagetype.width = 0;
-    imagetype.height = emSize;
+    imagetype.height = bitmap->emsize;
     imagetype.flags = FT_LOAD_DEFAULT;
 
     EnterCriticalSection(&freetype_cs);
-    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0)
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, bitmap->index, &glyph, NULL) == 0)
         pFT_Glyph_Get_CBox(glyph, FT_GLYPH_BBOX_PIXELS, &bbox);
     LeaveCriticalSection(&freetype_cs);
 
     /* flip Y axis */
-    ret->left = bbox.xMin;
-    ret->right = bbox.xMax;
-    ret->top = -bbox.yMax;
-    ret->bottom = -bbox.yMin;
+    bitmap->bbox.left = bbox.xMin;
+    bitmap->bbox.right = bbox.xMax;
+    bitmap->bbox.top = -bbox.yMax;
+    bitmap->bbox.bottom = -bbox.yMin;
 }
 
-void freetype_get_glyph_bitmap(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, const RECT *bbox, BYTE *buf)
+static BOOL freetype_get_aliased_glyph_bitmap(struct dwrite_glyphbitmap *bitmap, FT_Glyph glyph)
+{
+    const RECT *bbox = &bitmap->bbox;
+    int width = bbox->right - bbox->left;
+    int height = bbox->bottom - bbox->top;
+
+    if (glyph->format == FT_GLYPH_FORMAT_OUTLINE) {
+        FT_OutlineGlyph outline = (FT_OutlineGlyph)glyph;
+        const FT_Outline *src = &outline->outline;
+        FT_Bitmap ft_bitmap;
+        FT_Outline copy;
+
+        ft_bitmap.width = width;
+        ft_bitmap.rows = height;
+        ft_bitmap.pitch = bitmap->pitch;
+        ft_bitmap.pixel_mode = FT_PIXEL_MODE_MONO;
+        ft_bitmap.buffer = bitmap->buf;
+
+        /* Note: FreeType will only set 'black' bits for us. */
+        if (pFT_Outline_New(library, src->n_points, src->n_contours, &copy) == 0) {
+            pFT_Outline_Copy(src, &copy);
+            pFT_Outline_Translate(&copy, -bbox->left << 6, bbox->bottom << 6);
+            pFT_Outline_Get_Bitmap(library, &copy, &ft_bitmap);
+            pFT_Outline_Done(library, &copy);
+        }
+    }
+    else if (glyph->format == FT_GLYPH_FORMAT_BITMAP) {
+        FT_Bitmap *ft_bitmap = &((FT_BitmapGlyph)glyph)->bitmap;
+        BYTE *src = ft_bitmap->buffer, *dst = bitmap->buf;
+        int w = min(bitmap->pitch, (ft_bitmap->width + 7) >> 3);
+        int h = min(height, ft_bitmap->rows);
+
+        while (h--) {
+            memcpy(dst, src, w);
+            src += ft_bitmap->pitch;
+            dst += bitmap->pitch;
+        }
+    }
+    else
+        FIXME("format %x not handled\n", glyph->format);
+
+    return TRUE;
+}
+
+static BOOL freetype_get_aa_glyph_bitmap(struct dwrite_glyphbitmap *bitmap, FT_Glyph glyph)
+{
+    const RECT *bbox = &bitmap->bbox;
+    int width = bbox->right - bbox->left;
+    int height = bbox->bottom - bbox->top;
+    BOOL ret = FALSE;
+
+    if (glyph->format == FT_GLYPH_FORMAT_OUTLINE) {
+        FT_OutlineGlyph outline = (FT_OutlineGlyph)glyph;
+        const FT_Outline *src = &outline->outline;
+        FT_Bitmap ft_bitmap;
+        FT_Outline copy;
+
+        ft_bitmap.width = width;
+        ft_bitmap.rows = height;
+        ft_bitmap.pitch = bitmap->pitch;
+        ft_bitmap.pixel_mode = FT_PIXEL_MODE_GRAY;
+        ft_bitmap.buffer = bitmap->buf;
+
+        /* Note: FreeType will only set 'black' bits for us. */
+        if (pFT_Outline_New(library, src->n_points, src->n_contours, &copy) == 0) {
+            pFT_Outline_Copy(src, &copy);
+            pFT_Outline_Translate(&copy, -bbox->left << 6, bbox->bottom << 6);
+            pFT_Outline_Get_Bitmap(library, &copy, &ft_bitmap);
+            pFT_Outline_Done(library, &copy);
+        }
+    }
+    else if (glyph->format == FT_GLYPH_FORMAT_BITMAP) {
+        FT_Bitmap *ft_bitmap = &((FT_BitmapGlyph)glyph)->bitmap;
+        BYTE *src = ft_bitmap->buffer, *dst = bitmap->buf;
+        int w = min(bitmap->pitch, (ft_bitmap->width + 7) >> 3);
+        int h = min(height, ft_bitmap->rows);
+
+        while (h--) {
+            memcpy(dst, src, w);
+            src += ft_bitmap->pitch;
+            dst += bitmap->pitch;
+        }
+
+        ret = TRUE;
+    }
+    else
+        FIXME("format %x not handled\n", glyph->format);
+
+    return ret;
+}
+
+BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
 {
     FTC_ImageTypeRec imagetype;
+    BOOL ret = FALSE;
     FT_Glyph glyph;
 
-    imagetype.face_id = fontface;
+    imagetype.face_id = bitmap->fontface;
     imagetype.width = 0;
-    imagetype.height = emSize;
+    imagetype.height = bitmap->emsize;
     imagetype.flags = FT_LOAD_DEFAULT;
 
     EnterCriticalSection(&freetype_cs);
-    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0) {
-        int width = bbox->right - bbox->left;
-        int pitch = ((width + 31) >> 5) << 2;
-        int height = bbox->bottom - bbox->top;
-
-        if (glyph->format == FT_GLYPH_FORMAT_OUTLINE) {
-            FT_OutlineGlyph outline = (FT_OutlineGlyph)glyph;
-            const FT_Outline *src = &outline->outline;
-            FT_Bitmap ft_bitmap;
-            FT_Outline copy;
-
-            ft_bitmap.width = width;
-            ft_bitmap.rows = height;
-            ft_bitmap.pitch = pitch;
-            ft_bitmap.pixel_mode = FT_PIXEL_MODE_MONO;
-            ft_bitmap.buffer = buf;
-
-            /* Note: FreeType will only set 'black' bits for us. */
-            memset(buf, 0, height*pitch);
-            if (pFT_Outline_New(library, src->n_points, src->n_contours, &copy) == 0) {
-                pFT_Outline_Copy(src, &copy);
-                pFT_Outline_Translate(&copy, -bbox->left << 6, bbox->bottom << 6);
-                pFT_Outline_Get_Bitmap(library, &copy, &ft_bitmap);
-                pFT_Outline_Done(library, &copy);
-            }
-        }
-        else if (glyph->format == FT_GLYPH_FORMAT_BITMAP) {
-            FT_Bitmap *bitmap = &((FT_BitmapGlyph)glyph)->bitmap;
-            BYTE *src = bitmap->buffer, *dst = buf;
-            int w = min(pitch, (bitmap->width + 7) >> 3);
-            int h = min(height, bitmap->rows);
-
-            memset(buf, 0, height*pitch);
-
-            while (h--) {
-                memcpy(dst, src, w);
-                src += bitmap->pitch;
-                dst += pitch;
-            }
-        }
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, bitmap->index, &glyph, NULL) == 0) {
+        if (bitmap->type == DWRITE_TEXTURE_CLEARTYPE_3x1)
+            ret = freetype_get_aa_glyph_bitmap(bitmap, glyph);
         else
-            FIXME("format %d not handled\n", glyph->format);
+            ret = freetype_get_aliased_glyph_bitmap(bitmap, glyph);
     }
     LeaveCriticalSection(&freetype_cs);
+
+    return ret;
 }
 
 INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
@@ -599,6 +656,29 @@ INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
     return charmap_index;
 }
 
+INT32 freetype_get_glyph_advance(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
+{
+    FTC_ImageTypeRec imagetype;
+    FT_Glyph glyph;
+    INT32 advance;
+
+    imagetype.face_id = fontface;
+    imagetype.width = 0;
+    imagetype.height = emSize;
+    imagetype.flags = FT_LOAD_DEFAULT;
+    if (mode == DWRITE_MEASURING_MODE_NATURAL)
+        imagetype.flags |= FT_LOAD_NO_HINTING;
+
+    EnterCriticalSection(&freetype_cs);
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, index, &glyph, NULL) == 0)
+        advance = glyph->advance.x >> 16;
+    else
+        advance = 0;
+    LeaveCriticalSection(&freetype_cs);
+
+    return advance;
+}
+
 #else /* HAVE_FREETYPE */
 
 BOOL init_freetype(void)
@@ -650,15 +730,14 @@ INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 le
     return 0;
 }
 
-void freetype_get_glyph_bbox(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, BOOL nohint, RECT *ret)
+void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
 {
-    ret->left = ret->right = ret->top = ret->bottom = 0;
+    memset(&bitmap->bbox, 0, sizeof(bitmap->bbox));
 }
 
-void freetype_get_glyph_bitmap(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, const RECT *bbox, BYTE *buf)
+BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
 {
-    UINT32 size = (bbox->right - bbox->left)*(bbox->bottom - bbox->top);
-    memset(buf, 0, size);
+    return FALSE;
 }
 
 INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
@@ -667,4 +746,9 @@ INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
     return -1;
 }
 
+INT32 freetype_get_glyph_advance(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
+{
+    return 0;
+}
+
 #endif /* HAVE_FREETYPE */
