@@ -37,8 +37,6 @@
 #include "wine/wined3d.h"
 #include "wine/winedxgi.h"
 
-extern CRITICAL_SECTION dxgi_cs DECLSPEC_HIDDEN;
-
 /* Layered device */
 enum dxgi_device_layer_id
 {
@@ -76,12 +74,23 @@ const char *debug_dxgi_format(DXGI_FORMAT format) DECLSPEC_HIDDEN;
 
 DXGI_FORMAT dxgi_format_from_wined3dformat(enum wined3d_format_id format) DECLSPEC_HIDDEN;
 enum wined3d_format_id wined3dformat_from_dxgi_format(DXGI_FORMAT format) DECLSPEC_HIDDEN;
+void dxgi_sample_desc_from_wined3d(DXGI_SAMPLE_DESC *desc,
+        enum wined3d_multisample_type wined3d_type, unsigned int wined3d_quality) DECLSPEC_HIDDEN;
+void wined3d_sample_desc_from_dxgi(enum wined3d_multisample_type *wined3d_type,
+        unsigned int *wined3d_quality, const DXGI_SAMPLE_DESC *dxgi_desc) DECLSPEC_HIDDEN;
+HRESULT dxgi_get_private_data(struct wined3d_private_store *store,
+        REFGUID guid, UINT *data_size, void *data) DECLSPEC_HIDDEN;
+HRESULT dxgi_set_private_data(struct wined3d_private_store *store,
+        REFGUID guid, UINT data_size, const void *data) DECLSPEC_HIDDEN;
+HRESULT dxgi_set_private_data_interface(struct wined3d_private_store *store,
+        REFGUID guid, const IUnknown *object) DECLSPEC_HIDDEN;
 
 /* IDXGIFactory */
 struct dxgi_factory
 {
     IDXGIFactory1 IDXGIFactory1_iface;
     LONG refcount;
+    struct wined3d_private_store private_store;
     struct wined3d *wined3d;
     UINT adapter_count;
     IDXGIAdapter1 **adapters;
@@ -99,6 +108,7 @@ struct dxgi_device
     IWineDXGIDevice IWineDXGIDevice_iface;
     IUnknown *child_layer;
     LONG refcount;
+    struct wined3d_private_store private_store;
     struct wined3d_device *wined3d_device;
     IDXGIFactory1 *factory;
 };
@@ -111,6 +121,7 @@ struct dxgi_output
 {
     IDXGIOutput IDXGIOutput_iface;
     LONG refcount;
+    struct wined3d_private_store private_store;
     struct dxgi_adapter *adapter;
 };
 
@@ -122,6 +133,7 @@ struct dxgi_adapter
     IDXGIAdapter1 IDXGIAdapter1_iface;
     struct dxgi_factory *parent;
     LONG refcount;
+    struct wined3d_private_store private_store;
     UINT ordinal;
     IDXGIOutput *output;
 };
@@ -134,6 +146,7 @@ struct dxgi_swapchain
 {
     IDXGISwapChain IDXGISwapChain_iface;
     LONG refcount;
+    struct wined3d_private_store private_store;
     struct wined3d_swapchain *wined3d_swapchain;
 };
 
@@ -147,12 +160,12 @@ struct dxgi_surface
     IUnknown IUnknown_iface;
     IUnknown *outer_unknown;
     LONG refcount;
+    struct wined3d_private_store private_store;
     IDXGIDevice *device;
-
-    DXGI_SURFACE_DESC desc;
+    struct wined3d_resource *wined3d_resource;
 };
 
 HRESULT dxgi_surface_init(struct dxgi_surface *surface, IDXGIDevice *device,
-        IUnknown *outer, const DXGI_SURFACE_DESC *desc) DECLSPEC_HIDDEN;
+        IUnknown *outer, struct wined3d_resource *wined3d_resource) DECLSPEC_HIDDEN;
 
 #endif /* __WINE_DXGI_PRIVATE_H */
