@@ -1009,10 +1009,16 @@ NTSTATUS WINAPI IoGetDeviceInterfaces( const GUID *InterfaceClassGuid,
  */
 NTSTATUS  WINAPI IoGetDeviceObjectPointer( UNICODE_STRING *name, ACCESS_MASK access, PFILE_OBJECT *file, PDEVICE_OBJECT *device )
 {
+    static DEVICE_OBJECT stub_device;
+    static DRIVER_OBJECT stub_driver;
+
     FIXME( "stub: %s %x %p %p\n", debugstr_us(name), access, file, device );
 
+    stub_device.StackSize = 0x80; /* minimum value to appease SecuROM 5.x */
+    stub_device.DriverObject = &stub_driver;
+
     *file  = NULL;
-    *device  = NULL;
+    *device = &stub_device;
 
     return STATUS_SUCCESS;
 }
@@ -2298,3 +2304,19 @@ PLIST_ENTRY WINAPI ExfInterlockedRemoveHeadList(PLIST_ENTRY head, PKSPIN_LOCK lo
     FIXME("(%p %p) stub\n", head, lock);
     return ExInterlockedRemoveHeadList( head, lock );
 }
+
+/***********************************************************************
+ *           KeEnterCriticalRegion  (NTOSKRNL.EXE.@)
+ */
+void WINAPI KeEnterCriticalRegion(void)
+{
+    FIXME(": stub\n");
+}
+
+/***********************************************************************
+ *           KeLeaveCriticalRegion  (NTOSKRNL.EXE.@)
+ */
+void WINAPI KeLeaveCriticalRegion(void)
+{
+    FIXME(": stub\n");
+}
