@@ -117,6 +117,12 @@ enum {
     TOPMOST_FLOAT_INACTIVE_ALL,
 };
 
+enum {
+    GL_SURFACE_IN_FRONT_OPAQUE,
+    GL_SURFACE_IN_FRONT_TRANSPARENT,
+    GL_SURFACE_BEHIND,
+};
+
 enum {
     MACDRV_HOTKEY_SUCCESS,
     MACDRV_HOTKEY_ALREADY_REGISTERED,
@@ -149,6 +155,7 @@ extern int right_option_is_alt DECLSPEC_HIDDEN;
 extern int allow_immovable_windows DECLSPEC_HIDDEN;
 extern int cursor_clipping_locks_windows DECLSPEC_HIDDEN;
 extern int use_precise_scrolling DECLSPEC_HIDDEN;
+extern int gl_surface_mode DECLSPEC_HIDDEN;
 
 extern int macdrv_start_cocoa_app(unsigned long long tickcount) DECLSPEC_HIDDEN;
 extern void macdrv_window_rejected_focus(const struct macdrv_event *event) DECLSPEC_HIDDEN;
